dnl This was heavily modified from berkeley-db.m4 included with
dnl Subversion (r5118). Originally copyright (c) 2002-2003 CollabNet.

dnl   J2_LIB_OPENLDAP(major, minor, patch)
dnl
dnl   Search for a useable version of OpenLDAP in a number of
dnl   common places.  The installed lib must be no older than the
dnl   version given by MAJOR, MINOR and PATCH.
dnl
dnl   If we find a useable version, set CPPFLAGS and LIBS as
dnl   appropriate, and set the shell variable `j2_lib_openldap' to
dnl   `yes'.  Otherwise, set `j2_lib_openldap' to `no'.
dnl
dnl   This macro also checks for the `--with-openldap=PATH' flag;
dnl   if given, the macro will use the PATH specified, and the
dnl   configuration script will die if it can't find the library.
dnl
dnl   We cache the results of individual searches under particular
dnl   prefixes, not the overall result of whether we found OpenLDAP
dnl   That way, the user can re-run the configure script with
dnl   different --with-openldap switch values, without interference
dnl   from the cache.


AC_DEFUN(J2_LIB_OPENLDAP,
[
  openldap_version=$1.$2.$3
  dnl  Process the `with-openldap' switch. We set the variable `places' to
  dnl  either `search', meaning we should check in a list of typical places,
  dnl  or to a single place spec.
  dnl
  dnl  A `place spec' is either:
  dnl    - the string `std', indicating that we should look for headers and
  dnl      libraries in the standard places,
  dnl    - a directory prefix P, indicating we should look for headers in
  dnl      P/include and libraries in P/lib, or
  dnl    - a string of the form `HEADER:LIB', indicating that we should look
  dnl      for headers in HEADER and libraries in LIB.
  dnl 
  dnl  You'll notice that the value of the `--with-openldap' switch is a
  dnl  place spec.

  AC_ARG_WITH(openldap,
    AC_HELP_STRING(--with-openldap=PATH, [alternate location of OpenLDAP headers and libs]),
  [
    if test "$withval" = "yes"; then
      places=search
    else
      places="$withval"
    fi
  ],
  [
      places=search
  ])

    if test "$places" = "search"; then
      places="std"
    fi
    # Now `places' is guaranteed to be a list of place specs we should
    # search, no matter what flags the user passed.

    # Save the original values of the flags we tweak.
    J2_LIB_OPENLDAP_save_libs="$LIBS"
    J2_LIB_OPENLDAP_save_cppflags="$CPPFLAGS"

    # The variable `found' is the prefix under which we've found
    # OpenLDAP, or `not' if we haven't found it anywhere yet.
    found=not
    for place in $places; do

      LIBS="$J2_LIB_OPENLDAP_save_libs"
      CPPFLAGS="$J2_LIB_OPENLDAP_save_cppflags"
      case "$place" in
        "std" )
          description="the standard places"
        ;;
        *":"* )
          header="`echo $place | sed -e 's/:.*$//'`"
          lib="`echo $place | sed -e 's/^.*://'`"
	    CPPFLAGS="$CPPFLAGS -I$header"
        LIBS="$LIBS -L$lib -R$lib"
        description="$header and $lib"
        ;;
        * )
	    LIBS="$LIBS -L$place/lib -R$place/lib"
	    CPPFLAGS="$CPPFLAGS -I$place/include"
        description="$place"
        ;;
      esac

      # We generate a separate cache variable for each prefix
      # we search under.  That way, we avoid caching information that
      # changes if the user runs `configure' with a different set of
      # switches.
      changequote(,)
      cache_id="`echo j2_cv_lib_openldap_$1_$2_$3_in_${place} \
                   | sed -e 's/[^a-zA-Z0-9_]/_/g'`"
      changequote([,])
      dnl We can't use AC_CACHE_CHECK here, because that won't print out
      dnl the value of the computed cache variable properly.
      AC_MSG_CHECKING([for OpenLDAP in $description])
      AC_CACHE_VAL($cache_id,
        [
         J2_LIB_OPENLDAP_TRY($1, $2, $3)
           eval "$cache_id=$j2_have_openldap"
        ])
      result="`eval echo '$'$cache_id`"
      AC_MSG_RESULT($result)

      # If we found it, no need to search any more.
      if test "`eval echo '$'$cache_id`" = "yes"; then
        found="$place"
        break
      fi

      test "$found" != "not" && break
    done

    # Restore the original values of the flags we tweak.
    LIBS="$J2_LIB_OPENLDAP_save_libs"
    CPPFLAGS="$J2_LIB_OPENLDAP_save_cppflags"

    case "$found" in
      "not" )
	  AC_MSG_ERROR([Could not find OpenLDAP $openldap_version])
      j2_lib_openldap=no
      ;;
      "std" )
        J2_LDAP_INCLUDES=
        J2_LDAP_LIBS="-lldap -llber"
        j2_lib_openldap=yes
      ;;
      *":"* )
        header="`echo $found | sed -e 's/:.*$//'`"
        lib="`echo $found | sed -e 's/^.*://'`"
        J2_LDAP_INCLUDES="-I$header"
        J2_LDAP_LIBS="-L$lib -R$lib -lldap -llber"
        j2_lib_openldap=yes
      ;;
      * )
        J2_LDAP_INCLUDES="-I$found/include"
        J2_LDAP_LIBS="-L$found/lib -R$found/lib -lldap -llber"
        j2_lib_openldap=yes
      ;;
    esac
])


dnl   J2_LIB_OPENLDAP_TRY(major, minor, patch)
dnl
dnl   A subroutine of J2_LIB_OPENLDAP.
dnl
dnl   Check that a new-enough version of OpenLDAP is installed.
dnl   "New enough" means no older than the version given by MAJOR,
dnl   MINOR, FIX and PATCH.  The result of the test is not cached; no
dnl   messages are printed.
dnl
dnl   Set the shell variable `j2_have_openldap' to `yes' if we found
dnl   an appropriate version installed, or `no' otherwise.
dnl
dnl   This macro uses the OpenLDAP library function `ldap_get_option'
dnl   with the LDAP_OPT_API_INFO flag to find the version.  If the
dnl   library installed doesn't have this function, then this macro
dnl   assumes it is too old.


AC_DEFUN(J2_LIB_OPENLDAP_TRY,
  [
    j2_lib_openldap_try_save_libs="$LIBS"

    j2_check_openldap_major=$1
    j2_check_openldap_minor=$2
    j2_check_openldap_patch=$3

    LIBS="$LIBS -lldap -llber"

    AC_TRY_RUN(
      [
#include "ldap.h"
#include "lber.h"
#include <stdio.h>
main ()
{
  LDAPAPIInfo info;
  int major, minor, patch;

  /* get the version */
  info.ldapai_info_version = LDAP_API_INFO_VERSION;
  ldap_get_option(NULL, LDAP_OPT_API_INFO, &info);

  /* sanity; ensure that headers match the lib proper */
  if(info.ldapai_vendor_version != LDAP_VENDOR_VERSION)
    exit(1);
  
  /* extract version info */
  major = LDAP_VENDOR_VERSION / 10000;
  minor = (LDAP_VENDOR_VERSION / 100) % 100;
  patch = LDAP_VENDOR_VERSION % 100;
  
  /* ensure that the lib is good enough for us */
  if(major < $j2_check_openldap_major)
    exit(1);
  if(major > $j2_check_openldap_major)
    exit(0);

  if(minor < $j2_check_openldap_minor)
    exit(1);
  if(minor > $j2_check_openldap_minor)
    exit(0);

  if (patch >= $j2_check_openldap_patch)
    exit(0);
  else
    exit(1);
}
      ],
      [j2_have_openldap=yes],
      [j2_have_openldap=no],
      [j2_have_openldap=yes]
    )

  LIBS="$j2_lib_openldap_try_save_libs"
  ]
)
