/*
 * jabberd - Jabber Open Source Server
 * Copyright (c) 2002 Jeremie Miller, Thomas Muldowney,
 *                    Ryan Eatmon, Robert Norris
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

/*
 * !!! this is the same as util/inaddr.c - changes made here need to be made
 *     there too. is there anyway we can merge these without requiring mio to
 *     depend on util?
 */

#include "util.h"

/* parse a network address given as a string
 * return values:
 *     1 success
 *     0 address not valid
 */
int j_inet_pton(char *src, struct sockaddr_storage *dst)
{
#ifdef NO_INET_PTON
    struct sockaddr_in *sin;

    memset(dst, 0, sizeof(struct sockaddr_storage));
    sin = (struct sockaddr_in *)dst;
    
    if(inet_aton(src, &sin->sin_addr))
    {
    dst->ss_family = AF_INET;
    return 1;
    }

    return 0;
#else /* NO_INET_PTON */
    struct sockaddr_in *sin;
    struct sockaddr_in6 *sin6;

    memset(dst, 0, sizeof(struct sockaddr_storage));
    sin = (struct sockaddr_in *)dst;
    sin6 = (struct sockaddr_in6 *)dst;
    
    if(inet_pton(AF_INET, src, &sin->sin_addr) > 0)
    {
    dst->ss_family = AF_INET;
    return 1;
    }

    if(inet_pton(AF_INET6, src, &sin6->sin6_addr) > 0)
    {
    dst->ss_family = AF_INET6;
#ifdef SIN6_LEN
    sin6->sin6_len = sizeof(struct sockaddr_storage);
#endif /* SIN6_LEN */
    return 1;
    }

    return 0;
#endif /* NO_INET_PTON */
}

/* convert the network address in a sockaddr_storage structure to a string */
const char *j_inet_ntop(struct sockaddr_storage *src, char *dst, size_t size)
{
#ifdef NO_INET_NTOP
    char *tmp;
    struct sockaddr_in *sin;

    sin = (struct sockaddr_in *)src;

    /* if we don't have inet_ntop we only accept AF_INET
     * it's unlikely that we would have use for AF_INET6
     */
    if(src->ss_family != AF_INET)
    {
    return -1;
    }

    tmp = inet_ntoa(sin->sin_addr);

    if(!tmp || strlen(tmp)>=size)
    {
    return 0;
    }

    strncpy(dst, tmp, size);
    return 1;
#else /* NO_INET_NTOP */
    struct sockaddr_in *sin;
    struct sockaddr_in6 *sin6;

    sin = (struct sockaddr_in *)src;
    sin6 = (struct sockaddr_in6 *)src;

    switch(src->ss_family)
    {
    case AF_INET:
        return inet_ntop(AF_INET, &sin->sin_addr, dst, size);
    case AF_INET6:
        return inet_ntop(AF_INET6, &sin6->sin6_addr, dst, size);
    default:
        return NULL;
    }
#endif /* NO_INET_NTOP */
}

/* get the port out of a sockaddr_storage structure */
int j_inet_getport(struct sockaddr_storage *sa)
{
    struct sockaddr_in *sin;
    struct sockaddr_in6 *sin6;
    
    switch(sa->ss_family)
    {
    case AF_INET:
        sin = (struct sockaddr_in *)sa;
        return ntohs(sin->sin_port);
    case AF_INET6:
        sin6 = (struct sockaddr_in6 *)sa;
        return ntohs(sin6->sin6_port);
    default:
        return 0;
    }
}
