/*
 * jabberd - Jabber Open Source Server
 * Copyright (c) 2002 Jeremie Miller, Thomas Muldowney,
 *                    Ryan Eatmon, Robert Norris
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifdef HAVE_CONFIG_H
#   include <config.h>
#endif

#ifndef INCL_UTIL_COMPAT_H
#define INCL_UTIL_COMPAT_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * define the structures that could be missing in old libc implementations
 */

#ifndef PF_INET6
# define PF_INET6    10
#endif

#ifndef AF_INET6
# define AF_INET6    PF_INET6
#endif

#ifndef INET6_ADDRSTRLEN
# define INET6_ADDRSTRLEN 46
#endif

/* we have to define some structures if we compile on an old libc */
#ifdef NO_IN6_ADDR
struct in6_addr {
    uint8_t        s6_addr[16];    /* IPv6 address */
};
#endif /* NO_IN6_ADDR */

#ifdef NO_SOCKADDR_IN6
struct sockaddr_in6 {
#ifdef SIN6_LEN
    uint8_t        sin6_len;    /* length of this struct */
#endif /* SIN6_LEN */
    sa_family_t        sin6_family;    /* AF_INET6 */
    in_port_t        sin6_port;    /* transport layer port # */
    uint32_t        sin6_flowinfo;    /* IPv6 traffic class & flow info */
    struct in6_addr    sin6_addr;    /* IPv6 address */
    uint32_t        sin6_scope_id;    /* set of interfaces for a scope */
};
#endif /* NO_SOCKADDR_IN6 */

#ifdef NO_SOCKADDR_STORAGE
/* this definition is not fully compatible with RFC 2553,
 * but it is enough for us */
#define _SS_PADSIZE (128-sizeof(sa_family_t))
struct sockaddr_storage {
    sa_family_t        ss_family;    /* address family */
    char        __ss_pad[_SS_PADSIZE];
};
#endif /* NO_SOCKADDR_STORAGE */

#ifdef __cplusplus
}
#endif

#endif    /* INCL_UTIL_H */
